;; quick hack to make dot diagram of ASDF systems

(in-package #:asdf-extensions)

(defvar *ignore-errors?* nil)

(defun ensure-name (system)
  (intern (asdf::coerce-name system)
       (load-time-value (find-package :keyword))))

(defun safe-slot-value (thing name)
  (when (slot-boundp thing name)
    (slot-value thing name)))

(defgeneric system-dependencies* (system-definer system-name)
  (:documentation "Returns a list of the systems on which system-name depends directly."))

(defgeneric system-sub-systems* (system-definer system-name)
  (:documentation "Returns the complete list of the systems on which system-name depends; compare with system-dependencies*."))

(defun system-sub-systems (system-name)
  "Returns the complete list of the systems on which system-name depends; compare with system-dependencies."
  (system-sub-systems* :asdf (ensure-name system-name)))

(defun system-dependencies (system-name)
  "Returns a list of the systems on which system-name depends directly."
  (if *ignore-errors?*
      (ignore-errors      
     (system-dependencies* :asdf (ensure-name system-name)))
      (system-dependencies* :asdf (ensure-name system-name))))

(defun collect-system-dependencies
    (systems &key (depth nil) (ignore-errors? *ignore-errors?*))
  "What's in a name...? This form attempts to find and load all of
the referenced system files starting with any systems in systems.
It does not include the systems themselves unless they are sub-systems
of some other system in the list."
  (let ((systems-processed nil)
        (result nil)
     (*ignore-errors?* ignore-errors?)
     (maximum-depth depth))
    (labels ((compute-system-closure (system depth &optional (save? t))
            (when (and maximum-depth (> depth maximum-depth))
           (return-from compute-system-closure nil))
               (setf system (ensure-name system))
               (unless (member system systems-processed)
                 (push system systems-processed)
                 (when save?
                   (push system result))
                 (dolist (sub-system (system-dependencies system))
                   (when save?
                     (push sub-system result))
                   (compute-system-closure sub-system (1+ depth))))))
      (setf systems-processed nil)
      (dolist (system (if (consp systems) systems (list systems)))
        (compute-system-closure system 0 nil)))
    (remove-duplicates result)))

(defun map-system-dependencies (systems fn &key (depth nil))
  ;; this is weak in that it conses up the whole list and then maps that
  ;; but efficiency is probably not a big concern here...
  (mapc fn (collect-system-dependencies systems :depth depth)))

(defmethod system-sub-systems* ((system-definer (eql :asdf)) system-name)
  (collect-system-dependencies system-name))

(defmethod system-dependencies* ((system-definer (eql :asdf)) system-name)
  (loop for system in
       (cdadr (assoc 'asdf::load-op
               (safe-slot-value (asdf:find-system system-name)
                          'asdf::in-order-to)))
       collect (if *ignore-errors?*
          (or (ignore-errors (ensure-name system)) system)
          (ensure-name system))))

;;;;

(defun asdf-to-dot (system stream)
  (let* ((sub-systems (collect-system-dependencies system))
      (ids (make-hash-table :test 'equalp)))
    (let ((count 0))
      (setf (gethash system ids) count)
      (dolist (sub-system sub-systems)
     (setf (gethash sub-system ids) (incf count))))
    (flet ((system->id (system)
          (gethash system ids)))
      (format stream "~&digraph {~%graph [ label=\"~a\" ] ~%" system)
      (dolist (sub-system sub-systems)
     (format stream "~&~a [ label=\"~a\" ]; "
          (system->id sub-system) sub-system))
      (let ((processed nil))
     (labels ((do-it (child)
             (unless (member child processed)
               (push child processed)
               (dolist (sub-system (system-dependencies child))
                 (format stream "~& ~a -> ~a []; "
                      (system->id child) (system->id sub-system))
                 (do-it sub-system)))))
       (do-it system))
     (format stream "~&}")))))

#+(or)
(asdf-extensions::asdf-to-dot 'db.agraph *standard-output*)

#+(or)
(smu:with-new-file (stream "/tmp/db-agraph.dot") 
  (asdf-extensions::asdf-to-dot 'db.agraph stream))

#+(or)
(smu:with-new-file (stream "/tmp/db-agraph-tests.dot") 
  (asdf-extensions::asdf-to-dot 'db.agraph.tests stream))

#+(or)
(smu:with-new-file (stream "/tmp/service-daemon.dot") 
  (asdf-extensions::asdf-to-dot ':service-daemon stream))
